import logging

import pytest

from caravan.commands.base import BaseCommand


class TestCommand(BaseCommand):

    description = 'DESCRIPTION TEST'

    def setup_arguments(self, parser):
        parser.add_argument('-r', '--req', required=True)
        parser.add_argument('-o', '--opt')

    def run(self):
        print 'req=%s' % self.args.req
        print 'opt=%s' % self.args.opt
        print 'lvl=%s' % self.args.logging_level


def test_args(mocker, capsys):
    mocker.patch('sys.argv', ['PROG', '--req', 'REQ'])
    TestCommand.main()
    out, _ = capsys.readouterr()
    assert 'req=REQ' in out
    assert 'opt=None' in out

    mocker.patch('sys.argv', ['PROG', '--req', 'REQ', '--opt', 'OPT'])
    TestCommand.main()
    out, _ = capsys.readouterr()
    assert 'req=REQ' in out
    assert 'opt=OPT' in out


def test_args_logging_level(mocker, capsys):
    mocker.patch('sys.argv', ['PROG', '--req', 'REQ'])
    TestCommand.main()
    out, _ = capsys.readouterr()
    assert 'lvl=%s' % logging.WARNING in out

    mocker.patch('sys.argv', ['PROG', '--req', 'REQ', '--verbose'])
    TestCommand.main()
    out, _ = capsys.readouterr()
    assert 'lvl=%s' % logging.INFO in out

    mocker.patch('sys.argv', ['PROG', '--req', 'REQ', '--debug'])
    TestCommand.main()
    out, _ = capsys.readouterr()
    assert 'lvl=%s' % logging.DEBUG in out


LOGGING_CONFIG_FILE_DATA = """
[loggers]
keys = root

[logger_root]
level = DEBUG
handlers = console

[handlers]
keys = console

[handler_console]
class = StreamHandler
args = (sys.stderr,)
level = DEBUG
formatter = console

[formatters]
keys = console

[formatter_console]
format = %(message)s
"""


def test_args_logging_config(mocker, capsys, tmpdir):
    tmpdir.chdir()
    configfile = tmpdir.join('logging.conf')
    configfile.write(LOGGING_CONFIG_FILE_DATA)

    mocker.patch('sys.argv', ['PROG', '--req', 'REQ', '--logging-config',
                              'logging.conf'])
    TestCommand.main()


def test_args_missing(mocker, capsys):
    mocker.patch('sys.argv', ['PROG'])

    with pytest.raises(SystemExit):
        TestCommand.main()

    _, err = capsys.readouterr()
    assert 'PROG: error: argument -r/--req is required' in err


def test_args_help(mocker, capsys):
    mocker.patch('sys.argv', ['PROG', '--help'])

    with pytest.raises(SystemExit):
        TestCommand.main()

    out, _ = capsys.readouterr()
    assert TestCommand.description in out



from caravan.commands.signal import Command


def test_minimal(mocker, capsys):
    run_swf = mocker.patch('caravan.commands.signal.run_swf_command')
    mocker.patch('sys.argv', ['PROG', '-d', 'DOMAIN', '-i', 'ID', '-s', 'SIG'])

    Command.main()

    out, _ = capsys.readouterr()
    assert 'Signal sent.' in out

    run_swf.assert_called_once_with(
        'signal_workflow_execution',
        domain='DOMAIN',
        input=None,
        runId=None,
        signalName='SIG',
        workflowId='ID',
        )



from caravan.commands.terminate import Command


def run_and_get_mock(mocker, args):
    orig_args = ['PROG', '-d', 'DOMAIN', '-i', 'ID']
    args = orig_args + args
    mocker.patch('sys.argv', args)

    run_swf = mocker.patch('caravan.commands.terminate.run_swf_command')

    Command.main()

    return run_swf


def test_minimal(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=[])

    run_swf.assert_called_once_with(
        'terminate_workflow_execution',
        domain='DOMAIN',
        workflowId='ID',
        runId=None,
        reason=None,
        details=None,
        childPolicy=None,
        )

    out, _ = capsys.readouterr()
    assert 'Execution terminated.' in out


def test_runid(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--run-id', 'RUNID'])
    assert run_swf.call_args[1]['runId'] == 'RUNID'


def test_reason(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--reason', 'REASON'])
    assert run_swf.call_args[1]['reason'] == 'REASON'


def test_details(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--details', 'DETAILS'])
    assert run_swf.call_args[1]['details'] == 'DETAILS'


def test_child_policy(mocker, capsys):
    test_policy = Command.CHILD_POLICIES[0]
    run_swf = run_and_get_mock(mocker, args=['--child-policy', test_policy])
    assert run_swf.call_args[1]['childPolicy'] == test_policy



import datetime

from freezegun import freeze_time

from caravan.commands.list import Command


def run_and_get_mock(mocker, args):
    orig_args = ['PROG', '-d', 'DOMAIN']
    args = orig_args + args
    mocker.patch('sys.argv', args)

    run_swf = mocker.patch('caravan.commands.list.run_swf_command')
    run_swf.return_value = {'executionInfos': []}

    with freeze_time("2000-01-10"):
        Command.main()

    return run_swf


def test_minimal(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=[])

    start_time_filter = {
        'oldestDate': datetime.datetime(2000, 1, 9),
        'latestDate': datetime.datetime(2000, 1, 10),
        }

    run_swf.assert_called_once_with(
        'list_open_workflow_executions',
        domain='DOMAIN',
        startTimeFilter=start_time_filter,
        typeFilter=None,
        tagFilter=None,
        executionFilter=None,
        )

    out, _ = capsys.readouterr()
    assert 'No results.' in out


def test_type_filter(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['-n', 'NAME', '-v', 'VERSION'])
    expected = {'name': 'NAME', 'version': 'VERSION'}
    assert run_swf.call_args[1]['typeFilter'] == expected


def test_tag_filter(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--tag', 'TAG'])
    expected = {'tag': 'TAG'}
    assert run_swf.call_args[1]['tagFilter'] == expected


def test_id_filter(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--id', 'ID'])
    expected = {'workflowId': 'ID'}
    assert run_swf.call_args[1]['executionFilter'] == expected


# TODO: test formatter



from caravan.commands.start import Command


def run_and_get_mock(mocker, args):
    orig_args = ['PROG', '-d', 'DOMAIN', '-n', 'NAME', '-v', 'VERSION',
                 '-i', 'ID']
    args = orig_args + args
    mocker.patch('sys.argv', args)

    run_swf = mocker.patch('caravan.commands.start.run_swf_command')
    run_swf.return_value = {'runId': 'RUNID'}

    Command.main()

    return run_swf


def test_minimal(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=[])

    run_swf.assert_called_once_with(
        'start_workflow_execution',
        domain='DOMAIN',
        workflowId='ID',
        workflowType={'name': 'NAME', 'version': 'VERSION'},
        input=None,
        taskList=None,
        tagList=None,
        executionStartToCloseTimeout=None,
        taskStartToCloseTimeout=None,
        childPolicy=None,
        lambdaRole=None,
        )

    out, _ = capsys.readouterr()
    assert 'Execution started.' in out
    assert 'RunId: RUNID' in out


def test_task_list(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['-t', 'TASKLISTNAME'])
    assert run_swf.call_args[1]['taskList'] == {'name': 'TASKLISTNAME'}


def test_tag_list_1(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--tag', 'TAG1'])
    assert run_swf.call_args[1]['tagList'] == ['TAG1']


def test_tag_list_2(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--tag', 'TAG1 TAG2'])
    assert run_swf.call_args[1]['tagList'] == ['TAG1', 'TAG2']


def test_input(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--input', 'INPUT'])
    assert run_swf.call_args[1]['input'] == 'INPUT'


def test_execution_timeout(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--execution-timeout', '60'])
    assert run_swf.call_args[1]['executionStartToCloseTimeout'] == '60'


def test_task_timeout(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--task-timeout', '60'])
    assert run_swf.call_args[1]['taskStartToCloseTimeout'] == '60'


def test_child_policy(mocker, capsys):
    test_policy = Command.CHILD_POLICIES[0]
    run_swf = run_and_get_mock(mocker, args=['--child-policy', test_policy])
    assert run_swf.call_args[1]['childPolicy'] == test_policy


def test_lambda_role(mocker, capsys):
    run_swf = run_and_get_mock(mocker, args=['--lambda-role', 'ROLE'])
    assert run_swf.call_args[1]['lambdaRole'] == 'ROLE'
